/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <iostream>

#include <imgui.h>

#include <nostalgia/common/point.hpp>
#include <nostalgia/core/media.hpp>

#include "tilesheeteditor.hpp"

namespace nostalgia::core {

TileSheetEditor::TileSheetEditor(Context *ctx, const ox::String &path) {
	m_itemPath = path;
	const auto lastSlash = std::find(m_itemPath.crbegin(), m_itemPath.crend(), '/').offset();
	m_itemName = m_itemPath.substr(lastSlash + 1);
	m_ctx = ctx;
	// build shaders
	oxThrowError(m_pixels.buildShader());
	oxThrowError(m_pixelGrid.buildShader());
	oxRequireT(img, readObj<NostalgiaGraphic>(ctx, m_itemPath.c_str()));
	oxThrowError(readObj<NostalgiaPalette>(ctx, img->defaultPalette).moveTo(&m_pal));
	m_img = *img;
}

ox::String TileSheetEditor::itemName() const noexcept {
	return m_itemPath;
}

ox::String TileSheetEditor::itemDisplayName() const noexcept {
	return m_itemName;
}

void TileSheetEditor::exportFile() {
}

void TileSheetEditor::cut() {
}

void TileSheetEditor::copy() {
}

void TileSheetEditor::paste() {
}

void TileSheetEditor::draw(core::Context*) noexcept {
	const auto size = ImGui::GetContentRegionAvail();
	const auto sizei = common::Size(static_cast<int>(size.x), static_cast<int>(size.y));
	if (m_updated || m_framebuffer.width != sizei.width || m_framebuffer.height != sizei.height) {
		if (!m_framebuffer) {
			m_framebuffer = glutils::generateFrameBuffer(sizei.width, sizei.height);
		}
		m_pixels.initBufferSet(m_img, *m_pal);
		m_pixelGrid.initBufferSet(m_img);
	}
	glBindFramebuffer(GL_FRAMEBUFFER, m_framebuffer);
	glViewport(0, 0, sizei.width, sizei.height);
	glDraw();
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	ImGui::Image(reinterpret_cast<void*>(m_framebuffer.color.id), size, ImVec2(0, 1), ImVec2(1, 0));
	if (ImGui::IsItemHovered()) {
		const auto &io = ImGui::GetIO();
		const auto wheel = io.MouseWheel * 0.01f;
		const auto zoomMod = ox::defines::OS == ox::defines::OS::Darwin ? io.KeySuper : io.KeyCtrl;
		if (zoomMod && wheel != 0) {
			m_pixelSizeMod = ox::clamp(m_pixelSizeMod + wheel, 0.55f, 2.f);
			m_pixels.setPixelSizeMod(m_pixelSizeMod);
			m_pixelGrid.setPixelSizeMod(m_pixelSizeMod);
			m_updated = true;
		}
		if (io.MouseClicked[0]) {
			auto clickPos = io.MouseClickedPos[0];
			const auto &winPos = ImGui::GetWindowPos();
			clickPos.x -= winPos.x;
			clickPos.y -= winPos.y;
			std::cout << clickPos.x - 8 << ", " << clickPos.y - 31 << '\n';
		}
	}
}

void TileSheetEditor::glDraw() noexcept {
	// clear screen
	constexpr Color32 bgColor = 0x717d7e;
	glClearColor(redf(bgColor), greenf(bgColor), bluef(bgColor), 1.f);
	glClear(GL_COLOR_BUFFER_BIT);
	m_pixels.draw(m_updated);
	m_pixelGrid.draw(m_updated);
	m_updated = false;
	//oxAssert(glGetError() == 0, "There was an OpenGL error");
}

void TileSheetEditor::saveItem() {
}

}
